const { __ } = wp.i18n; // Import __() from wp.i18n
const { Component } = wp.element;
const { MediaUpload, RichText, InspectorControls, PanelColorSettings } = wp.editor;
const { RangeControl, PanelBody } = wp.components;

import { defaultItem, getStyles } from './block';
import { InspectorContainer, ContainerEdit } from '../commonComponents/container/container';
import { Plus } from '../commonComponents/icons/plus';

/**
 * Keys for new blocks
 * @type {number}
 */
let key = 0;

/**
 * The edit function describes the structure of your block in the context of the editor.
 * This represents what the editor will render when the block is used.
 *
 * The "edit" property must be a valid function.
 * @param {Object} props - attributes
 * @returns {Node} rendered component
 */
export default class Edit extends Component {
    state = {
        activeSubBlock: -1,
    };

    /**
     * Add a new item to list with default fields
     */
    addItem = () => {
        key++;
        this.props.setAttributes( {
            items: [ ...this.props.attributes.items, {
                ...defaultItem,
                title: defaultItem.title + ' ' + ( key ),
                key: 'new ' + new Date().getTime(),
            } ],
        } );
        setTimeout( () => {
            const element = document.querySelector( '.owl-carousel' );
            element.scrollLeft = element.scrollWidth;
        } );
    };

    /**
     * Change any property of item
     * @param {string} property - editable field
     * @param {string} value - for field
     * @param {number} index - of items array
     * @param {boolean} withMutation - in some cases we should avoid mutation for force rerender component
     */
    onChangePropertyItem = ( property, value, index, withMutation = false ) => {
        const items = withMutation ? [ ...this.props.attributes.items ] : this.props.attributes.items;
        if ( ! items[ index ] || typeof items[ index ][ property ] !== 'string' ) {
            return;
        }
        items[ index ][ property ] = value;
        this.props.setAttributes( { items: items } );
    };

    /**
     * Set active sub block to edit icon
     * @param {number} index from sub block array
     */
    setActiveSubBlock = ( index ) => {
        if ( this.state.activeSubBlock !== index ) {
            this.setState( { activeSubBlock: index } );
        }
    };

    /**
     * Remove item
     * It also add default item if we remove all elements from array
     * @param {number} index - of item
     */
    removeItem = ( index ) => {
        const items = [ ...this.props.attributes.items ];
        if ( items.length === 1 ) {
            this.props.setAttributes( { items: [ defaultItem ] } );
        } else {
            items.splice( index, 1 );
            this.props.setAttributes( { items: items } );
        }
    };

    render() {
        const {
            className,
            attributes,
            setAttributes,
            isSelected,
        } = this.props;

        const { vars, kenzapContanerStyles } = getStyles( attributes );
        return (
            <div>
                <InspectorControls>
                    <PanelBody
                        title={ __( 'General', 'kenzap-features' ) }
                        initialOpen={ false }
                    >
                        <RangeControl
                            label={ __( 'Main title size', 'kenzap-features' ) }
                            value={ attributes.mainTitleSize }
                            onChange={ ( mainTitleSize ) => setAttributes( { mainTitleSize } ) }
                            min={ 20 }
                            max={ 130 }
                        />
                        <RangeControl
                            label={ __( 'Icons size', 'kenzap-features' ) }
                            value={ attributes.iconSize }
                            onChange={ ( iconSize ) => setAttributes( { iconSize } ) }
                            min={ 50 }
                            max={ 130 }
                        />
                        <RangeControl
                            label={ __( 'Title size', 'kenzap-features' ) }
                            value={ attributes.titleSize }
                            onChange={ ( titleSize ) => setAttributes( { titleSize } ) }
                            min={ 10 }
                            max={ 40 }
                        />
                        <RangeControl
                            label={ __( 'Description size', 'kenzap-features' ) }
                            value={ attributes.descriptionSize }
                            onChange={ ( descriptionSize ) => setAttributes( { descriptionSize } ) }
                            min={ 10 }
                            max={ 30 }
                        />
                        <PanelColorSettings
                            title={ __( 'Colors' ) }
                            initialOpen={ false }
                            colorSettings={ [
                                {
                                    value: attributes.mainTitleColor,
                                    onChange: ( value ) => {
                                        return setAttributes( { mainTitleColor: value } );
                                    },
                                    label: __( 'Main title color', 'kenzap-features' ),
                                },
                                {
                                    value: attributes.titleColor,
                                    onChange: ( value ) => {
                                        return setAttributes( { titleColor: value } );
                                    },
                                    label: __( 'Title color', 'kenzap-features' ),
                                },
                                {
                                    value: attributes.descriptionColor,
                                    onChange: ( value ) => {
                                        return setAttributes( { descriptionColor: value } );
                                    },
                                    label: __( 'Description color', 'kenzap-features' ),
                                },
                            ] }
                        />
                    </PanelBody>
                    <InspectorContainer
                        setAttributes={ setAttributes }
                        { ...attributes }
                        withPadding
                        withWidth100
                        withBackground
                    />
                </InspectorControls>
                <div className={ className ? className : '' }>
                    <ContainerEdit
                        className={ `kenzap-featured-list-2 block-${ attributes.blockUniqId } ${ isSelected ? 'selected' : '' } ` }
                        attributes={ attributes }
                        style={ {
                            ...vars,
                        } }
                        withBackground
                        withPadding
                    >
                        <div className="kenzap-container" style={ kenzapContanerStyles }>

                            <RichText
                                tagName="h2"
                                placeholder={ __( 'Title', 'kenzap-features' ) }
                                value={ attributes.mainTitle }
                                style={ {
                                    color: attributes.mainTitleColor,
                                    lineHeight: 1.4,
                                } }
                                onChange={ ( value ) => setAttributes( { mainTitle: value } ) }
                                onSplit={ () => null }
                            />
                            <div className="owl-carousel">
                                { attributes.items && attributes.items.map( ( item, index ) => (
                                    <div
                                        key={ item.key }
                                        className="featured-box"
                                        role="presentation"
                                        onClick={ () => this.setActiveSubBlock( index ) }
                                    >
                                        <button className="remove" onClick={ () => this.removeItem( index ) }>
                                            <span className="dashicons dashicons-no" />
                                        </button>

                                        { item.iconMediaUrl ? (
                                            <MediaUpload
                                                onSelect={ ( media ) => {
                                                    this.onChangePropertyItem( 'iconMediaId', media.id, index );
                                                    this.onChangePropertyItem( 'iconMediaUrl', media.url, index, true );
                                                } }
                                                value={ item.iconMediaId }
                                                allowedTypes={ [ 'image', 'image/svg+xml' ] }
                                                render={ ( props ) => (
                                                    <img
                                                        src={ item.iconMediaUrl }
                                                        alt={ item.title.replace( /<(?:.|\n)*?>/gm, '' ) }
                                                        style={ {
                                                            height: `${ attributes.iconSize }px`,
                                                            cursor: 'pointer',
                                                            position: 'relative',
                                                            zIndex: 10,
                                                        } }
                                                        onClick={ props.open }
                                                        role="presentation"
                                                    />
                                                ) }
                                            />
                                        ) : (
                                            <div
                                                className="addIcon"
                                                style={ {
                                                    height: `${ attributes.iconSize }px`,
                                                } }>
                                                <MediaUpload
                                                    onSelect={ ( media ) => {
                                                        this.onChangePropertyItem( 'iconMediaId', media.id, index );
                                                        this.onChangePropertyItem( 'iconMediaUrl', media.url, index, true );
                                                    } }
                                                    value={ item.iconMediaId }
                                                    allowedTypes={ [ 'image', 'image/svg+xml' ] }
                                                    render={ ( props ) => (
                                                        <button onClick={ props.open }>
                                                            { __( 'Upload/Choose icon', 'kenzap-features' ) }
                                                        </button>
                                                    ) }
                                                />
                                            </div>
                                        ) }

                                        <RichText
                                            tagName="h3"
                                            placeholder={ __( 'Title', 'kenzap-features' ) }
                                            value={ item.title }
                                            onChange={ ( value ) => this.onChangePropertyItem( 'title', value, index, true ) }
                                            style={ {
                                                color: attributes.titleColor,
                                            } }
                                        />
                                        <RichText
                                            tagName="p"
                                            placeholder={ __( 'Description', 'kenzap-features' ) }
                                            value={ item.description }
                                            onChange={ ( value ) => this.onChangePropertyItem( 'description', value, index, true ) }
                                            style={ {
                                                color: attributes.descriptionColor,
                                            } }
                                        />
                                    </div>
                                ) ) }
                            </div>
                        </div>
                        <div className="editPadding"/> 
                        <button className="kenzap-add" onClick={ this.addItem }>
                            <Plus />
                            <span/>{ __( 'Add new feature', 'kenzap-features' ) }
                        </button>
                    </ContainerEdit>
                </div>
            </div>
        );
    }
}
